from _typeshed import Incomplete

class cached_property:
    func: Incomplete
    def __init__(self, func) -> None: ...
    def __get__(self, obj, cls): ...

class class_property:
    func: Incomplete
    def __init__(self, func) -> None: ...
    def __get__(self, instance, cls): ...

class class_cached_property:
    func: Incomplete
    def __init__(self, func) -> None: ...
    def __get__(self, obj, cls): ...
