from typing import Final

symbols: Final[list[str]]
SELECT: Final = "SELECT"
INSERT: Final = "INSERT"
UPDATE: Final = "UPDATE"
DELETE: Final = "DELETE"
SELECT_FOR_UPDATE: Final = "SELECT_FOR_UPDATE"
FROM: Final = "FROM"
INNER_JOIN: Final = "INNER_JOIN"
LEFT_JOIN: Final = "LEFT_JOIN"
WHERE: Final = "WHERE"
GROUP_BY: Final = "GROUP_BY"
HAVING: Final = "HAVING"
UNION: Final = "UNION"
INTERSECT: Final = "INTERSECT"
EXCEPT: Final = "EXCEPT"
ORDER_BY: Final = "ORDER_BY"
LIMIT: Final = "LIMIT"
ASC: Final = "ASC"
DESC: Final = "DESC"
DISTINCT: Final = "DISTINCT"
ALL: Final = "ALL"
AGGREGATES: Final = "AGGREGATES"
AS: Final = "AS"
COUNT: Final = "COUNT"
SUM: Final = "SUM"
MIN: Final = "MIN"
MAX: Final = "MAX"
AVG: Final = "AVG"
TABLE: Final = "TABLE"
COLUMN: Final = "COLUMN"
PARAM: Final = "PARAM"
VALUE: Final = "VALUE"
AND: Final = "AND"
OR: Final = "OR"
NOT: Final = "NOT"
EQ: Final = "EQ"
NE: Final = "NE"
LT: Final = "LT"
LE: Final = "LE"
GT: Final = "GT"
GE: Final = "GE"
IS_NULL: Final = "IS_NULL"
IS_NOT_NULL: Final = "IS_NOT_NULL"
LIKE: Final = "LIKE"
NOT_LIKE: Final = "NOT_LIKE"
BETWEEN: Final = "BETWEEN"
NOT_BETWEEN: Final = "NOT_BETWEEN"
IN: Final = "IN"
NOT_IN: Final = "NOT_IN"
EXISTS: Final = "EXISTS"
NOT_EXISTS: Final = "NOT_EXISTS"
ROW: Final = "ROW"
ADD: Final = "ADD"
SUB: Final = "SUB"
MUL: Final = "MUL"
DIV: Final = "DIV"
POW: Final = "POW"
NEG: Final = "NEG"
ABS: Final = "ABS"
UPPER: Final = "UPPER"
LOWER: Final = "LOWER"
CONCAT: Final = "CONCAT"
STRIN: Final = "STRIN"
SUBSTR: Final = "SUBSTR"
LENGTH: Final = "LENGTH"
TRIM: Final = "TRIM"
LTRIM: Final = "LTRIM"
RTRIM: Final = "RTRIM"
REPLACE: Final = "REPLACE"
CASE: Final = "CASE"
COALESCE: Final = "COALESCE"
TO_INT: Final = "TO_INT"
RANDOM: Final = "RANDOM"
DATE: Final = "DATE"
YEAR: Final = "YEAR"
MONTH: Final = "MONTH"
DAY: Final = "DAY"
HOUR: Final = "HOUR"
MINUTE: Final = "MINUTE"
SECOND: Final = "SECOND"
TODAY: Final = "TODAY"
NOW: Final = "NOW"
DATE_ADD: Final = "DATE_ADD"
DATE_SUB: Final = "DATE_SUB"
DATETIME_ADD: Final = "DATETIME_ADD"
DATETIME_SUB: Final = "DATETIME_SUB"
