
from setuptools import setup

setup(package_data={'pony-stubs': ['__init__.pyi', 'converting.pyi', 'flask/__init__.pyi', 'flask/example/__init__.pyi', 'flask/example/app.pyi', 'flask/example/config.pyi', 'flask/example/models.pyi', 'flask/example/views.pyi', 'options.pyi', 'orm/__init__.pyi', 'orm/asttranslation.pyi', 'orm/core.pyi', 'orm/dbapiprovider.pyi', 'orm/dbproviders/__init__.pyi', 'orm/dbproviders/cockroach.pyi', 'orm/dbproviders/mysql.pyi', 'orm/dbproviders/oracle.pyi', 'orm/dbproviders/postgres.pyi', 'orm/dbproviders/sqlite.pyi', 'orm/dbschema.pyi', 'orm/decompiling.pyi', 'orm/examples/__init__.pyi', 'orm/examples/alessandro_bug.pyi', 'orm/examples/bottle_example.pyi', 'orm/examples/bug_ben.pyi', 'orm/examples/compositekeys.pyi', 'orm/examples/demo.pyi', 'orm/examples/estore.pyi', 'orm/examples/inheritance1.pyi', 'orm/examples/numbers.pyi', 'orm/examples/session01.pyi', 'orm/examples/university1.pyi', 'orm/examples/university2.pyi', 'orm/integration/__init__.pyi', 'orm/integration/bottle_plugin.pyi', 'orm/ormtypes.pyi', 'orm/serialization.pyi', 'orm/sqlbuilding.pyi', 'orm/sqlsymbols.pyi', 'orm/sqltranslation.pyi', 'py23compat.pyi', 'thirdparty/__init__.pyi', 'thirdparty/decorator.pyi', 'utils/__init__.pyi', 'utils/properties.pyi', 'utils/utils.pyi', 'METADATA.toml', 'py.typed']})
